@@ -21,6 +21,7 @@ from batchgenerators.utilities.file_and_folder_operations import *
 from nnunet.network_architecture.neural_network import SegmentationNetwork
 from sklearn.model_selection import KFold
 from torch import nn
+from torch.cuda.amp import GradScaler, autocast
 from torch.optim.lr_scheduler import _LRScheduler
 
 matplotlib.use("agg")
@@ -35,11 +36,7 @@ import torch.backends.cudnn as cudnn
 from abc import abstractmethod
 from datetime import datetime
 from tqdm import trange
-
-try:
-    from apex import amp
-except ImportError:
-    amp = None
+from nnunet.utilities.to_torch import maybe_to_torch, to_cuda
 
 
 class NetworkTrainer(object):
@@ -60,7 +57,7 @@ class NetworkTrainer(object):
         - predict_test_case
         """
         self.fp16 = fp16
-        self.amp_initialized = False
+        self.amp_grad_scaler = None
 
         if deterministic:
             np.random.seed(12345)
@@ -270,14 +267,17 @@ class NetworkTrainer(object):
             optimizer_state_dict = None
 
         self.print_to_log_file("saving checkpoint...")
-        torch.save({
+        save_this = {
             'epoch': self.epoch + 1,
             'state_dict': state_dict,
             'optimizer_state_dict': optimizer_state_dict,
             'lr_scheduler_state_dict': lr_sched_state_dct,
             'plot_stuff': (self.all_tr_losses, self.all_val_losses, self.all_val_losses_tr_mode,
-                           self.all_val_eval_metrics)},
-            fname)
+                           self.all_val_eval_metrics)}
+        if self.amp_grad_scaler is not None:
+            save_this['amp_grad_scaler'] = self.amp_grad_scaler.state_dict()
+
+        torch.save(save_this, fname)
         self.print_to_log_file("done, saving took %.2f seconds" % (time() - start_time))
 
     def load_best_checkpoint(self, train=True):
@@ -323,10 +323,10 @@ class NetworkTrainer(object):
         """
         pass
 
-    def load_checkpoint_ram(self, saved_model, train=True):
+    def load_checkpoint_ram(self, checkpoint, train=True):
         """
         used for if the checkpoint is already in ram
-        :param saved_model:
+        :param checkpoint:
         :param train:
         :return:
         """
@@ -337,33 +337,32 @@ class NetworkTrainer(object):
         curr_state_dict_keys = list(self.network.state_dict().keys())
         # if state dict comes form nn.DataParallel but we use non-parallel model here then the state dict keys do not
         # match. Use heuristic to make it match
-        for k, value in saved_model['state_dict'].items():
+        for k, value in checkpoint['state_dict'].items():
             key = k
             if key not in curr_state_dict_keys and key.startswith('module.'):
                 key = key[7:]
             new_state_dict[key] = value
 
-        # if we are fp16, then we need to reinitialize the network and the optimizer. Otherwise amp will throw an error
         if self.fp16:
-            self.network, self.optimizer, self.lr_scheduler = None, None, None
-            self.initialize_network()
-            self.initialize_optimizer_and_scheduler()
+            self._maybe_init_amp()
+            if 'amp_grad_scaler' in checkpoint.keys():
+                self.amp_grad_scaler.load_state_dict(checkpoint['amp_grad_scaler'])
 
         self.network.load_state_dict(new_state_dict)
-        self.epoch = saved_model['epoch']
+        self.epoch = checkpoint['epoch']
         if train:
-            optimizer_state_dict = saved_model['optimizer_state_dict']
+            optimizer_state_dict = checkpoint['optimizer_state_dict']
             if optimizer_state_dict is not None:
                 self.optimizer.load_state_dict(optimizer_state_dict)
 
-            if self.lr_scheduler is not None and hasattr(self.lr_scheduler, 'load_state_dict') and saved_model[
+            if self.lr_scheduler is not None and hasattr(self.lr_scheduler, 'load_state_dict') and checkpoint[
                 'lr_scheduler_state_dict'] is not None:
-                self.lr_scheduler.load_state_dict(saved_model['lr_scheduler_state_dict'])
+                self.lr_scheduler.load_state_dict(checkpoint['lr_scheduler_state_dict'])
 
             if issubclass(self.lr_scheduler.__class__, _LRScheduler):
                 self.lr_scheduler.step(self.epoch)
 
-        self.all_tr_losses, self.all_val_losses, self.all_val_losses_tr_mode, self.all_val_eval_metrics = saved_model[
+        self.all_tr_losses, self.all_val_losses, self.all_val_losses_tr_mode, self.all_val_eval_metrics = checkpoint[
             'plot_stuff']
 
         # after the training is done, the epoch is incremented one more time in my old code. This results in
@@ -379,19 +378,11 @@ class NetworkTrainer(object):
             self.all_val_losses_tr_mode = self.all_val_losses_tr_mode[:self.epoch]
             self.all_val_eval_metrics = self.all_val_eval_metrics[:self.epoch]
 
-        self.amp_initialized = False
         self._maybe_init_amp()
 
     def _maybe_init_amp(self):
-        # we use fp16 for training only, not inference
-        if self.fp16 and torch.cuda.is_available():
-            if not self.amp_initialized:
-                if amp is not None:
-                    self.network, self.optimizer = amp.initialize(self.network, self.optimizer, opt_level="O1")
-                    self.amp_initialized = True
-                else:
-                    raise RuntimeError("WARNING: FP16 training was requested but nvidia apex is not installed. "
-                                       "Install it from https://github.com/NVIDIA/apex")
+        if self.fp16 and self.amp_grad_scaler is None and torch.cuda.is_available():
+                self.amp_grad_scaler = GradScaler()
 
     def plot_network_architecture(self):
         """
@@ -616,33 +607,39 @@ class NetworkTrainer(object):
         data = data_dict['data']
         target = data_dict['target']
 
-        if not isinstance(data, torch.Tensor):
-            data = torch.from_numpy(data).float()
-        if not isinstance(target, torch.Tensor):
-            target = torch.from_numpy(target).float()
+        data = maybe_to_torch(data)
+        target = maybe_to_torch(target)
 
         if torch.cuda.is_available():
-            data = data.cuda(non_blocking=True)
-            target = target.cuda(non_blocking=True)
+            data = to_cuda(data)
+            target = to_cuda(target)
 
         self.optimizer.zero_grad()
-        output = self.network(data)
-        del data
-        l = self.loss(output, target)
+
+        if self.fp16:
+            with autocast():
+                output = self.network(data)
+                del data
+                l = self.loss(output, target)
+
+            if do_backprop:
+                self.amp_grad_scaler.scale(l).backward()
+                self.amp_grad_scaler.step(self.optimizer)
+                self.amp_grad_scaler.update()
+        else:
+            output = self.network(data)
+            del data
+            l = self.loss(output, target)
+
+            if do_backprop:
+                l.backward()
+                self.optimizer.step()
 
         if run_online_evaluation:
             self.run_online_evaluation(output, target)
 
         del target
 
-        if do_backprop:
-            if not self.fp16 or amp is None or not torch.cuda.is_available():
-                l.backward()
-            else:
-                with amp.scale_loss(l, self.optimizer) as scaled_loss:
-                    scaled_loss.backward()
-            self.optimizer.step()
-
         return l.detach().cpu().numpy()
 
     def run_online_evaluation(self, *args, **kwargs):
